<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Models\Portfolio;
use App\Models\PortfolioCategory;
use App\Models\PortfolioPhoto;
use Illuminate\Http\Request;
use Carbon\Carbon;
use Illuminate\Validation\Rule;
use Image;
use Illuminate\Support\Str;

class PortfolioController extends Controller
{
    public function __construct()
    {
        $this->middleware('auth');
    }

    public function index()
    {
        return view('admin.portfolio', [
            'portfolios' => Portfolio::get(),
        ]);
    }

    public function portfolio_add()
    {
        return view('admin.portfolio_add',[
            'portfolio_categories' => PortfolioCategory::get()
        ]);
    }

    public function insert(Request $request)
    {
        if(env('PROJECT_MODE') == 0) {
            return redirect()->back()->with('error', env('PROJECT_NOTIFICATION'));
        }

        $request->validate([
            'photo' => 'required|file|image',
            'banner' => 'required|file|image',
            'name' => 'required',
            'slug' => 'unique:portfolios',
            'short_content' => 'required',
            'full_content' => 'required',
            'category_id' => 'required',
        ]);

        if ($request->slug == '') {
            $slug = Str::slug($request->name);
        } else {
            $slug = Str::slug($request->slug);
        }
        $count = Portfolio::where('slug', $slug)->count();
        if ($count != 0) {
            return back()->with('error', A_ERROR_SLUG_EXIST);
        }

        if ($request->hasFile('photo')) {
            $file = $request->file('photo');
            $ext = $file->getClientOriginalExtension();
            $filename1 = 'portfolio_'.md5(mt_rand(11111111,99999999)).'.'.$ext;
            $file->move(public_path('uploads'),$filename1);
        }

        if ($request->hasFile('banner')) {
            $file = $request->file('banner');
            $ext = $file->getClientOriginalExtension();
            $filename2 = 'portfolio_banner_'.md5(mt_rand(11111111,99999999)).'.'.$ext;
            $file->move(public_path('uploads'),$filename2);
        }

        Portfolio::create([
            'name' => $request->name,
            'slug' => $slug,
            'short_content' => $request->short_content,
            'full_content' => $request->full_content,
            'client_name' => $request->client_name,
            'client_company' => $request->client_company,
            'start_date' => $request->start_date,
            'end_date' => $request->end_date,
            'website' => $request->website,
            'cost' => $request->cost,
            'client_comment' => $request->client_comment,
            'category_id' => $request->category_id,
            'photo' => $filename1,
            'banner' => $filename2,
            'show_on_home' => $request->show_on_home,
            'meta_title' => $request->meta_title,
            'meta_description' => $request->meta_description
        ]);

        return Redirect()->route('admin.portfolio')->with('success', A_SUCCESS_DATA_ADD);
    }

    public function edit_page($id)
    {
        return view('admin.portfolio_edit', [
            'portfolio' => Portfolio::find($id),
            'portfolio_categories' => PortfolioCategory::all(),
        ]);
    }

    public function edit(Request $request)
    {
        if(env('PROJECT_MODE') == 0) {
            return redirect()->back()->with('error', env('PROJECT_NOTIFICATION'));
        }

        $id = $request->id;

        $request->validate([
            'name' => 'required',
            'slug' => [Rule::unique('portfolios')->ignore($id),],
            'short_content' => 'required',
            'full_content' => 'required',
            'category_id' => 'required',
        ]);

        if ($request->slug == '') {
            $slug = Str::slug($request->name);
        } else {
            $slug = Str::slug($request->slug);
        }

        $count = Portfolio::where('slug', $slug)->where('id', '!=', $id)->count();
        if ($count != 0) {
            return back()->with('error', A_ERROR_SLUG_EXIST);
        }

        $obj = Portfolio::findOrFail($id);
        if ($request->hasFile('photo')) {
            $request->validate([
                'photo' => 'image'
            ]);
            $file = $request->file('photo');
            $ext = $file->getClientOriginalExtension();
            @unlink(public_path('uploads/'.$obj->photo));
            $filename1 = 'portfolio_'.md5(mt_rand(11111111,99999999)).'.'.$ext;
            $file->move(public_path('uploads'),$filename1);
            $obj->photo = $filename1;
        }
        if ($request->hasFile('banner')) {
            $request->validate([
                'banner' => 'image'
            ]);
            $file = $request->file('banner');
            $ext = $file->getClientOriginalExtension();
            @unlink(public_path('uploads/'.$obj->banner));
            $filename2 = 'portfolio_banner_'.md5(mt_rand(11111111,99999999)).'.'.$ext;
            $file->move(public_path('uploads'),$filename2);
            $obj->banner = $filename2;
        }

        $obj->name = $request->name;
        $obj->slug = $slug;
        $obj->short_content = $request->short_content;
        $obj->full_content = $request->full_content;
        $obj->client_name = $request->client_name;
        $obj->client_company = $request->client_company;
        $obj->start_date = $request->start_date;
        $obj->end_date = $request->end_date;
        $obj->website = $request->website;
        $obj->cost = $request->cost;
        $obj->client_comment = $request->client_comment;
        $obj->category_id = $request->category_id;
        $obj->show_on_home = $request->show_on_home;
        $obj->meta_title = $request->meta_title;
        $obj->meta_description = $request->meta_description;
        $obj->update();

        return Redirect()->route('admin.portfolio')->with('success', A_SUCCESS_DATA_UPDATE);
    }

    public function delete($id)
    {
        if(env('PROJECT_MODE') == 0) {
            return redirect()->back()->with('error', env('PROJECT_NOTIFICATION'));
        }

        $photo = Portfolio::find($id)->photo;
        unlink(public_path('uploads/' . $photo));
        $banner = Portfolio::find($id)->banner;
        unlink(public_path('uploads/' . $banner));
        Portfolio::find($id)->forceDelete();

        $portfolio_photos = PortfolioPhoto::where('portfolio_id',$id)->get();
        foreach($portfolio_photos as $item) {
            unlink(public_path('uploads/'.$item->photo));
        }
        PortfolioPhoto::find($id)->forceDelete();

        return Redirect()->route('admin.portfolio')->with('success', A_SUCCESS_DATA_DELETE);
    }

    public function gallery_section($id)
    {
        $portfolio_photos = PortfolioPhoto::where('portfolio_id',$id)->get();
        $portfolio_single = Portfolio::where('id',$id)->first();
        $portfolio_id = $id;
        return view('admin.portfolio_gallery', compact('portfolio_photos','portfolio_id','portfolio_single'));
    }

    public function gallery_store(Request $request)
    {
        if(env('PROJECT_MODE') == 0) {
            return redirect()->back()->with('error', env('PROJECT_NOTIFICATION'));
        }

        $request->validate([
            'photo' => 'required|file|image'
        ]);
        if ($request->hasFile('photo')) {
            $file = $request->file('photo');
            $ext = $file->getClientOriginalExtension();
            $filename1 = 'portfolio_photo_'.md5(mt_rand(11111111,99999999)).'.'.$ext;
            $file->move(public_path('uploads'),$filename1);
        }
        PortfolioPhoto::create([
            'portfolio_id' => $request->id,
            'photo' => $filename1
        ]);

        return redirect()->back()->with('success', A_SUCCESS_DATA_ADD);
    }

    public function gallery_delete($id)
    {
        if(env('PROJECT_MODE') == 0) {
            return redirect()->back()->with('error', env('PROJECT_NOTIFICATION'));
        }
        
        $photo = PortfolioPhoto::find($id)->photo;
        unlink(public_path('uploads/' . $photo));
        PortfolioPhoto::find($id)->forceDelete();
        return Redirect()->back()->with('success', A_SUCCESS_DATA_DELETE);
    }
}
